`timescale 1ns / 1ps

module uart_rx(clk,rstn,rxd,baud_set,data_byte,rx_done
    );
input clk;
input rstn;
input rxd;
input [2:0] baud_set;
output reg [7:0] data_byte;
output reg rx_done;

parameter SYS_CLK=50_000_000;

reg uart_rx_reg1;
reg uart_rx_reg2;
wire edge_fall;
reg uart_state;


reg [7:0] bps_clk_cnt;
reg [2:0] start_bit;


reg [2:0] data_byte_sum [7:0];
reg [2:0] stop_bit;



reg [15:0] bps_DR;

reg [15:0] div_cnt;



always @(posedge clk or posedge rstn)
    begin
        if (!rstn)
        begin
        uart_rx_reg1<=1'b0;
        uart_rx_reg2<=1'b0;
        end
        else
            begin
            uart_rx_reg1<=rxd;
            uart_rx_reg2<=uart_rx_reg1;
            end
    end
assign  edge_fall =!uart_rx_reg1&uart_rx_reg2 ;




always @(posedge clk or posedge rstn)
    begin
        if (!rstn)
            uart_state<=1'b0;
        else if (edge_fall) uart_state<=1'b1;
        else if(rx_done||((bps_clk_cnt==8'd12)&&(start_bit>2))||((bps_clk_cnt==8'd155)&&(stop_bit<3))) uart_state<=1'b0;
        else uart_state<=uart_state;
    end

always @(posedge clk or posedge rstn)
    begin
        if (!rstn)
        begin
    bps_DR<=16'd0;
        end
        else begin
            case(baud_set)
            0:bps_DR<=SYS_CLK/9600/16-1;
            1:bps_DR<=SYS_CLK/19200/16-1;
            2:bps_DR<=SYS_CLK/38400/16-1;
            3:bps_DR<=SYS_CLK/57600/16-1;
            4:bps_DR<=SYS_CLK/115200/16-1;
            default :bps_DR<=SYS_CLK/9600/16-1;
            endcase
            end
    end


always @(posedge clk or posedge rstn) begin
    if (!rstn) begin
        div_cnt<=16'd0;
    end
    else if (uart_state) begin
    if(div_cnt==bps_DR)
        div_cnt<=16'd0;
    else
        div_cnt<=div_cnt+1'b1;
    end
    else
    div_cnt<=16'd0;
    end

reg bps_clk;
always @(posedge clk or posedge rstn)
        begin
            if (!rstn)
            begin
            bps_clk<=1'b0;
            end
            else if (div_cnt==16'd1)
                    begin
                    bps_clk<=1'b1;
                    end
                else
                    begin
                    bps_clk<=1'b0;
                    end
        end



always @(posedge clk or posedge rstn)
    begin
        if (!rstn)
        begin
        bps_clk_cnt<=8'd0;
        end
        else if ((bps_clk_cnt==8'd159)||((bps_clk_cnt==8'd12)&&(start_bit>2)))
               bps_clk_cnt<=8'd0;
        else if(bps_clk)
        bps_clk_cnt<=bps_clk_cnt+1'b1;
        else begin
        bps_clk_cnt<=bps_clk_cnt;
        end
    end

always @(posedge clk or posedge rstn)
    begin
        if (!rstn)
        begin
        rx_done<=1'b0;
        end
        else if (bps_clk_cnt==8'd159)
            begin
            rx_done<=1'b1;
            end
            else begin
            rx_done<=1'b0;
            end
    end


always @(posedge clk or posedge rstn)
    begin
        if (!rstn)
        begin
        start_bit<=3'd0;
        data_byte_sum[0]<=3'd0;
        data_byte_sum[1]<=3'd0;
        data_byte_sum[2]<=3'd0;
        data_byte_sum[3]<=3'd0;
        data_byte_sum[4]<=3'd0;
        data_byte_sum[5]<=3'd0;
        data_byte_sum[6]<=3'd0;
        data_byte_sum[7]<=3'd0;
        stop_bit<=3'd0;
        end
        else if (bps_clk)
            begin
                case(bps_clk_cnt)
                0:
                    begin
                        start_bit<=3'd0;
                        data_byte_sum[0]<=3'd0;
                        data_byte_sum[1]<=3'd0;
                        data_byte_sum[2]<=3'd0;
                        data_byte_sum[3]<=3'd0;
                        data_byte_sum[4]<=3'd0;
                        data_byte_sum[5]<=3'd0;
                        data_byte_sum[6]<=3'd0;
                        data_byte_sum[7]<=3'd0;
                        stop_bit<=3'd0;
                    end
                6,7,8,9,10,11:start_bit<=start_bit+uart_rx_reg2;
                22,23,24,25,26,27:data_byte_sum[0]<=data_byte_sum[0]+uart_rx_reg2;
                38,39,40,41,42,43:data_byte_sum[1]<=data_byte_sum[0]+uart_rx_reg2;
                54,55,56,57,58,59:data_byte_sum[2]<=data_byte_sum[0]+uart_rx_reg2;
                70,71,72,73,74,75:data_byte_sum[3]<=data_byte_sum[0]+uart_rx_reg2;
                86,87,88,89,90,91:data_byte_sum[4]<=data_byte_sum[0]+uart_rx_reg2;
                102,103,104,105,106,107:data_byte_sum[5]<=data_byte_sum[0]+uart_rx_reg2;
                118,119,120,121,122,123:data_byte_sum[6]<=data_byte_sum[0]+uart_rx_reg2;
                134,135,136,137,138,139:data_byte_sum[7]<=data_byte_sum[0]+uart_rx_reg2;
                150,151,152,153,154,155:stop_bit<=stop_bit+uart_rx_reg2;
                default:
                    begin
                start_bit<=start_bit;
                data_byte_sum[0]<=data_byte_sum[0];
                data_byte_sum[1]<=data_byte_sum[1];
                data_byte_sum[2]<=data_byte_sum[2];
                data_byte_sum[3]<=data_byte_sum[3];
                data_byte_sum[4]<=data_byte_sum[4];
                data_byte_sum[5]<=data_byte_sum[5];
                data_byte_sum[6]<=data_byte_sum[6];
                data_byte_sum[7]<=data_byte_sum[7];
                stop_bit<=stop_bit;
                    end
                endcase
            end
    end

always @(posedge clk or posedge rstn)
        if (!rstn) data_byte<=8'd0;
        else if (bps_clk_cnt==8'd159)
            begin
            data_byte[0]<=data_byte_sum[0][2];
            data_byte[1]<=data_byte_sum[1][2];
            data_byte[2]<=data_byte_sum[2][2];
            data_byte[3]<=data_byte_sum[3][2];
            data_byte[4]<=data_byte_sum[4][2];
            data_byte[5]<=data_byte_sum[5][2];
            data_byte[6]<=data_byte_sum[6][2];
            data_byte[7]<=data_byte_sum[7][2];
            end



endmodule
