`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:
// Engineer:
//
// Create Date: 2022/12/05 14:47:31
// Design Name:
// Module Name: uart_tx
// Project Name:
// Target Devices:
// Tool Versions:
// Description:
//
// Dependencies:
//
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
//
//////////////////////////////////////////////////////////////////////////////////


module uart_tx (
    input clk,
    input rstn,
    input send_en,
    input [2:0] baud_set,
    input [7:0] data_byte,
    output reg uart_txd,
    output reg tx_done,
    output reg uart_state
);

  reg [15:0] bps_DR;
  reg [15:0] div_cnt;
  reg bps_clk;
  reg [3:0] bps_clk_cnt;
  reg [7:0] data_byte_reg;



  parameter START_BIT = 1'b0;
  parameter STOP_BIT = 1'b1;

  always @(posedge clk or negedge rstn) begin
    if (!rstn) uart_state <= 1'b0;
    else if (send_en) uart_state <= 1'b1;
    else if (bps_clk_cnt == 4'd11) uart_state <= 1'b0;
    else uart_state <= uart_state;
  end



  always @(posedge clk or negedge rstn) begin
    if (!rstn) data_byte_reg <= 8'd0;
    else if (send_en) data_byte_reg <= data_byte;
    else data_byte_reg <= data_byte_reg;
  end



  always @(posedge clk or negedge rstn) begin
    if (!rstn) bps_DR = 16'd0;
    else begin
      case (baud_set)
        0: bps_DR <= 16'd5207;
        1: bps_DR <= 16'd2603;
        2: bps_DR <= 16'd1301;
        3: bps_DR <= 16'd867;
        4: bps_DR <= 16'd433;
        default: bps_DR <= 16'd5207;
      endcase
    end
  end

  always @(posedge clk or negedge rstn) begin
    if (!rstn) div_cnt <= 16'd0;
    else if (uart_state)
      begin
        if (div_cnt == bps_DR) div_cnt <= 16'd0;
        else div_cnt <= div_cnt + 1'b1;
      end
    else div_cnt <= 16'd0;
  end

  always @(posedge clk or negedge rstn) begin
    if (!rstn) bps_clk <= 1'b0;
    else if (div_cnt == 16'd1) bps_clk <= 1'b1;
    else bps_clk <= 0;
  end

  always @(posedge clk or negedge rstn) begin
    if (!rstn) bps_clk_cnt <= 0;
    else if (bps_clk_cnt == 4'd11) bps_clk_cnt <= 4'd0;
    else if (bps_clk) bps_clk_cnt <= bps_clk_cnt + 1'b1;
    else bps_clk_cnt <= bps_clk_cnt;
  end

  always @(posedge clk or negedge rstn) begin
    if (!rstn) tx_done <= 1'b0;
    else if (bps_clk_cnt == 4'd11) tx_done <= 1'b1;
    else tx_done <= 1'b0;
  end


  always @(posedge clk or negedge rstn) begin
    if (!rstn) uart_txd = 1'b1;
    else begin
      case (bps_clk_cnt)
        0: uart_txd <= 1'b1;
        1: uart_txd <= START_BIT;
        2: uart_txd <= data_byte_reg[0];
        3: uart_txd <= data_byte_reg[1];
        4: uart_txd <= data_byte_reg[2];
        5: uart_txd <= data_byte_reg[3];
        6: uart_txd <= data_byte_reg[4];
        7: uart_txd <= data_byte_reg[5];
        8: uart_txd <= data_byte_reg[6];
        9: uart_txd <= data_byte_reg[7];
        10: uart_txd <= STOP_BIT;
        default: uart_txd <= 1'b1;
      endcase
    end
  end

endmodule
